'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Creates an Aggregated Trigger Data Object with totals and avg properties
 *  updated when _computeData is called
 * @constructor
 */
function AggregatedData() {
    this.count                  = 0;
    this.male                   = 0;
    this.female                 = 0;
    this.uncertain_gender       = 0;
    this.age                    = 0;
    this.smile                  = 0;
}

AggregatedData.prototype._computeData = function( data ) {
    
    try {

        var count_people = (this._checkForOldAggregatedSubmission(data))? data.detection_count : 0;
        var males, females, uncertain_gender, total_age, total_smile;
        var males = females = uncertain_gender = total_age = total_smile =  0;
        if (count_people) {
            data.aggregated_data.forEach(function(detection){
                // Count totals gender
                if(detection.gender === 'm') males++;
                else if(detection.gender === 'f') females++;
                else uncertain_gender++;
                // Aggregate age and smile for avg
                total_age += detection.age;
                total_smile += detection.smile_viewing;
            });

            //Compute counter, avg and percentage 
            this.count = count_people;
            this.male = Math.floor((males/count_people)*100).toFixed(0);
            this.female = Math.floor((females/count_people)*100).toFixed(0);
            this.uncertain_gender =  Math.floor((uncertain_gender/count_people)*100).toFixed(0);
            this.age = (total_age/count_people).toFixed(0);
            this.smile = (total_smile/count_people).toFixed(0);

        } else {
            // If not detections, set as 0 all values
            this._resetAggregatedData();
        } 

    } catch (error) {
        console.log("AggregatedData.update Error : " + error);
    }
}


AggregatedData.prototype._resetAggregatedData = function( data ) {
    this.count              = 0;
    this.male               = 0;
    this.female             = 0;
    this.age                = 0;
    this.smile              = 0;
    this.uncertain_gender   = 0;
}

AggregatedData.prototype._checkForOldAggregatedSubmission = function( data ) { 
    if((Date.now() - data.aggregator_freq) > Date.parse(data.submission_time+ " GMT")) return false;
    else return true;
}
