'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Creates an Aggregated detection by id Object
 * @constructor
 * @param {Object} data 
 */
function AggregatedDetection(data) {

        this.id                                  = data.id || 0;
        this.timestamp                           = data.timestamp || "";
        this.age                                 = data.age || 0;
        this.gender                              = this.parseGenderValue(data.gender);
        this.impressions                         = this.parseBooleanToHuminizedValue(data.impressions);
        this.viewers                             = this.parseBooleanToHuminizedValue(data.views);
        this.attention_sessions                  = data.attention_sessions || 0;
        this.average_attention_time_per_sessions = data.average_attention_time || 0;
        this.attention_time                      = data.attention_time || 0;
        this.dwell_time                          = data.dwell_time || 0;
        this.total_detection_time                = data.total_detection_time || 0;
        this.smile_viewing                       = data.smile_viewing || 0;
        this.smile_maximum                       = data.smile_maximum || 0;
        this.face_mask                           = this.parseBooleanToHuminizedValue(data.face_mask);
}

AggregatedDetection.prototype.parseGenderValue = function( value ) {
        
        if(value === 'm') return "Male";
        else if(value === 'f') return "Female";
        else  if(value === 'u') return 'Uncertain';
        return undefined;
        
} 

AggregatedDetection.prototype.parseBooleanToHuminizedValue = function( value ) {
        
        if(value <= 0 ) return "No";
        else if(value > 0) return "Yes";
        return undefined;
        
} 