'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/
/**
 * Aggregate detections to collection
 */
DeepsightToolkit.prototype.proccesRawDataRequest = function (data) {
    var self = this;

    try {

        data.people.forEach(function (person, idx) {

            var detection;
            if (self._rawDetections.has(parseInt(person.id)))
                detection = self._rawDetections.get(parseInt(person.id));
            else if (self._lostDetections.has(parseInt(person.id))) {
                detection = self._lostDetections.get(parseInt(person.id));
                self._lostDetections.delete(parseInt(person.id));
            } else {
                detection = new RawDetection(parseInt(person.id));
            }
            
            detection.update(person, data.timestamp);

            if (self._isRecordingSession) {
                var rawSessionDetection;
                if (self._rawSessionDetections.has(parseInt(person.id)))
                    rawSessionDetection = self._rawSessionDetections.get(parseInt(person.id));
                else {
                    rawSessionDetection = new RawSessionDetection(parseInt(person.id));
                }
                rawSessionDetection.update(detection, parseInt(self.maskThreshold));
                self._rawSessionDetections.set(parseInt(person.id), rawSessionDetection);
            }

            self._rawDetections.set(parseInt(person.id), detection);
        });


        if (data.people.length) {
            var frameIds = data.people.map(function (person, idx) { return person.id; });
            this._updateLostDetections(frameIds);
        } else {
            this._updateLostDetections([]);
        }

    } catch (error) {
        console.log("DeepsightToolkit.prototype.proccesRawDataRequest Error : " + error);
    }
}

/**
 * Update frame detections collection
 */
DeepsightToolkit.prototype._updateRawDetections = function (frameIds, rawDetections) {
    try {
        var frameFaces = Array.from(rawDetections)
            .filter(function (obj, idx) {
                return (frameIds.indexOf(obj.id) > -1);
            });

        this.realTimeAggregatedData._computeData(frameFaces, parseInt(this.maskThreshold));

        this.emit("realTimeAggregatedDataChanged", [this.realTimeAggregatedData]);
        // //Emit trigger for IF of Updated RealTime Crowd Data
        this.emit("RealtimeAggregatedData", [this.realTimeAggregatedData.count,
                                            this.realTimeAggregatedData.maleCount,
                                            this.realTimeAggregatedData.malePercentage,
                                            this.realTimeAggregatedData.femaleCount,
                                            this.realTimeAggregatedData.femalePercentage,
                                            this.realTimeAggregatedData.age,
                                            this.realTimeAggregatedData.smile,
                                            this.realTimeAggregatedData.withMask,
                                            this.realTimeAggregatedData.countViewer,
                                            this.realTimeAggregatedData.maleCountViewer,
                                            this.realTimeAggregatedData.malePercentageViewer,
                                            this.realTimeAggregatedData.femaleCountViewer,
                                            this.realTimeAggregatedData.femalePercentageViewer,
                                            this.realTimeAggregatedData.ageViewer,
                                            this.realTimeAggregatedData.smileViewer,
                                            this.realTimeAggregatedData.withMaskViewer]);
    } catch (error) {
        console.log("DeepsightToolkit.prototype._updateRawDetections : " + error);
    }
}

/**
 * Update frame detections collection
 */
DeepsightToolkit.prototype._updateLostDetections = function (frameIds) {

    try {
        var self = this;
        var _lostDetections;

        if (this._rawDetections.size) {
            _lostDetections = Array.from(this._rawDetections.values())
                .filter(function (obj, idx) {
                    return !(frameIds.indexOf(obj.id) > -1);
                });

            _lostDetections.forEach(function (detection) {
                if (self._rawDetections.has(parseInt(detection.id))) {
                    self._lostDetections.set(parseInt(detection.id), detection);
                    self._rawDetections.delete(parseInt(detection.id));
                }
            });
        }
        this._cleanOldLostDetections();
        
        setTimeout(function(){
            self._updateRawDetections(frameIds, self._rawDetections.values());
        }, 1)

    } catch (error) {
        console.error("DeepsightToolkitAggregationData.prototype._updateLostDetections : " + error);
    }
}


DeepsightToolkit.prototype._cleanOldLostDetections = function () {

    try {

        var self = this;
        this._lostDetections.forEach(function (lostDetection, key) {
            if (lostDetection.last_timestamp * 1000 < Date.now() - 1500)
                self._lostDetections.delete(key);
        });

    } catch (error) {
        console.error("DeepsightToolkitAggregationData.prototype._cleanOldLostDetections : " + error);
    }
}