'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Update Get request body after than ipAddress / port / endpoint changed
 */
DeepsightToolkit.prototype.updateURLRequests = function () {
    this.aggregatedRequestURL = "http://" + this.ipAddress + ":" + this.port + "/JSONAGGR";
    this.rawRequestURL = "http://" + this.ipAddress + ":" + this.port + "/JSON";
    this.emit('getRequestChanged', [this.getRequestURL]);
};

/**
 * Change IpAddress
 * @param {String} ipAddress
 */
DeepsightToolkit.prototype.setIpAddress = function (ipAddress) {
    if (this.ipAddress != ipAddress) {
        this.ipAddress = ipAddress;
        this.emit('ipAddressChanged', [this.ipAddress]);
        this.updateURLRequests();
    }
};


/**
 * Change port
 * @param {Number} port
 */
DeepsightToolkit.prototype.setPort = function (port) {
    if (this.port != port) {
        this.port = port;
        this.emit('portChanged', [this.port]);
        this.updateURLRequests();
    }
};

/**
 * Change endpoint
 * @param {String} endpoint
 */
DeepsightToolkit.prototype.setEndpoint = function (endpoint) {
    if (this.endpoint != endpoint) {
        this.endpoint = endpoint;
        this.emit('endpointChanged', [this.endpoint]);
        this.updateGetRequest();
    }
};

/**
 * Change maskThreshold
 * @param {Number} maskThreshold
 */
DeepsightToolkit.prototype.setMaskThreshold = function (maskThreshold) {
    if (this.maskThreshold != maskThreshold) {
        this.maskThreshold = maskThreshold;
        this.emit('maskThresholdChanged', [this.maskThreshold]);
    }
}


/**
 * Change isAllowedToTriggerSessionAggregatedDataPerId
 * @param {Boolean} isAllowedToTriggerSessionAggregatedDataPerId
 */
DeepsightToolkit.prototype.setIsAllowedToTriggerSessionAggregatedDataPerId = function (isAllowedToTriggerSessionAggregatedDataPerId) {
    if (this.isAllowedToTriggerSessionAggregatedDataPerId != isAllowedToTriggerSessionAggregatedDataPerId) {
        this.isAllowedToTriggerSessionAggregatedDataPerId = isAllowedToTriggerSessionAggregatedDataPerId;
        this.emit('isAllowedToTriggerSessionAggregatedDataPerIdChanged', [this.isAllowedToTriggerSessionAggregatedDataPerId]);
    }
}

DeepsightToolkit.prototype.startAggregatedData = function () {

    var self = this;
    if (!this._isRequestingAggregatedData) {
        this._isRequestingAggregatedData = true;
        this.pullerAggregatedData();
    }

}


DeepsightToolkit.prototype.stopAggregatedData = function () {
    this._isRequestingAggregatedData = false;
}

/**
 *
 *  updateAggregatedData call a GET request with the address: self.getRequestURL
 *  get data from JSONAGGR endpoint from Deepsight Toolkit and trigger an event in 
 *  IF with the computed Aggregated Trigger Data  
 * 
 */
DeepsightToolkit.prototype.updateAggregatedData = function () {

    try {

        var self = this;
        var headers = {};
        this.httpService.get(this.aggregatedRequestURL, headers, {
            "success": function (result) {
                setTimeout(function () {
                    try {
                        var resultJSON = JSON.parse(result);

                        if (resultJSON.aggregated_data !== undefined) {
                            self.aggregatedData._computeData(resultJSON);
                            //Emit trigger for IF changed readonly properties of Aggregated Trigger Data
                            self.emit("aggregatedDataChanged", [self.aggregatedData]);
                            //Emit trigger for IF of Updated Aggregated Trigger Data
                            self.emit("AggregatedData", [
                                self.aggregatedData.count,
                                self.aggregatedData.male,
                                self.aggregatedData.female,
                                self.aggregatedData.uncertain_gender,
                                self.aggregatedData.age,
                                self.aggregatedData.smile
                            ]);
                        }
                    } catch (error) {
                        console.log("DeepsightToolkit.prototype.updateAggregatedData Success Callback Catch Error: " + error);
                    }
                }, 1);

            },
            "error": function (errorMessage) {
                setTimeout(function () {
                    console.log("DeepsightToolkit.prototype.updateAggregatedData Error callback: " + errorMessage);
                }, 1);

            }
        });
    } catch (error) {
        console.log("DeepsightToolkit.prototype.updateAggregatedData: Catch Error: " + error);
    }
};



/**
 *
 *  proccesAggregatedDataRequest
 * 
 */
DeepsightToolkit.prototype.proccesAggregatedDataRequest = function (response) {
    var self = this;
    try {
        if (response.aggregated_data != undefined) {
            if (response.aggregated_data.length) {
                var aggregatedReportingDatasIDs = [];
                response.aggregated_data.forEach(function (detection) {
                    var _detection = new AggregatedDetection(detection);
                    self._aggregatedDetections.set(parseInt(_detection.id), _detection);
                    aggregatedReportingDatasIDs.push(parseInt(_detection.id));
                });

                var oldAggregatedDetectionsIds = Array.from(self._aggregatedDetections.keys()).filter(function (aggrDataId) {
                    return aggregatedReportingDatasIDs.indexOf(aggrDataId) == -1;
                });

                oldAggregatedDetectionsIds.forEach(oldAggregatedDetectionId => {
                    var oldAggregagatedDetection = self._aggregatedDetections.get(oldAggregatedDetectionId);
                    self.emit("AggregatedReportingData",[ oldAggregagatedDetection.id,
                                                          oldAggregagatedDetection.timestamp,
                                                          oldAggregagatedDetection.age,
                                                          oldAggregagatedDetection.gender,
                                                          oldAggregagatedDetection.impressions,
                                                          oldAggregagatedDetection.viewers,
                                                          oldAggregagatedDetection.attention_sessions,
                                                          oldAggregagatedDetection.average_attention_time_per_sessions,
                                                          oldAggregagatedDetection.attention_time,
                                                          oldAggregagatedDetection.dwell_time,
                                                          oldAggregagatedDetection.total_detection_time,
                                                          oldAggregagatedDetection.smile_viewing,
                                                          oldAggregagatedDetection.smile_maximum,
                                                          oldAggregagatedDetection.face_mask]);
                                                          
                    self._aggregatedDetections.delete(oldAggregatedDetectionId);

                });

            } else {
                self.clearAggregatedDetections();
            }
        }
    } catch (error) {
        console.log("DeepsightToolkit.prototype.proccesAggregatedDataRequest: Catch Error: " + error);
    }
};

DeepsightToolkit.prototype.clearAggregatedDetections = function () {
    var self = this;
    try {
        Array.from(self._aggregatedDetections.values()).forEach(function (detection) {
            self.emit("AggregatedReportingData", [ detection.id,
                                                    detection.timestamp,
                                                    detection.age,
                                                    detection.gender,
                                                    detection.impressions,
                                                    detection.viewers,
                                                    detection.attention_sessions,
                                                    detection.average_attention_time_per_sessions,
                                                    detection.attention_time,
                                                    detection.dwell_time,
                                                    detection.total_detection_time,
                                                    detection.smile_viewing,
                                                    detection.smile_maximum,
                                                    detection.face_mask]);
            
            
        });
        self._aggregatedDetections.clear();
    } catch (error) {
        console.log("DeepsightToolkit.prototype.clearAggregatedDetections: Catch Error: " + error);
    }

}

/**
 * Notify that aggregatedData object from Deepsight Toolkit Object changed
 */
DeepsightToolkit.prototype.notifyThatAggregatedDataChanged = function () {
    this.emit("aggregatedDataChanged", [this.aggregatedData]);
};

DeepsightToolkit.prototype.startRawData = function () {

    var self = this;
    if (!this._isRequestingRawData) {
        this._isRequestingRawData = true;
        this.pullerRawData();
    }

}

DeepsightToolkit.prototype.stopRawData = function () {

    this._isRequestingRawData = false;
    this._resetTimeoutRequestQueue();
}



DeepsightToolkit.prototype.startRecordingSession = function () {
    var self = this;
    if (!this._isRecordingSession) {
        this._isRecordingSession = true;
        setTimeout(function () {
            self.rawSessionAggregatedData._resetRawSessionAggregatedData();
        }, 1000);

    }
}

DeepsightToolkit.prototype.stopRecordingSession = function () {

    if (this._isRecordingSession) {
        this._isRecordingSession = false;
        this.triggerSessionData();
    }
}

DeepsightToolkit.prototype.triggerSessionData = function () {
    try {
        var self = this;
        this.rawSessionAggregatedData._computeData(this._rawSessionDetections);
        this.emit("rawSessionAggregatedDataChanged", [this.rawSessionAggregatedData]);
        this.emit("SessionAggregatedData", [this.rawSessionAggregatedData.count,
                                            this.rawSessionAggregatedData.maleCount,
                                            this.rawSessionAggregatedData.malePercentage,
                                            this.rawSessionAggregatedData.femaleCount,
                                            this.rawSessionAggregatedData.femalePercentage,
                                            this.rawSessionAggregatedData.uncertainCount,
                                            this.rawSessionAggregatedData.uncertainPercentage,
                                            this.rawSessionAggregatedData.impressionsCount,
                                            this.rawSessionAggregatedData.viewersCount,
                                            this.rawSessionAggregatedData.age,
                                            this.rawSessionAggregatedData.smileViewing,
                                            this.rawSessionAggregatedData.detectionDuration,
                                            this.rawSessionAggregatedData.dwellDuration,
                                            this.rawSessionAggregatedData.attentionDuration,
                                            this.rawSessionAggregatedData.maleDwellDuration,
                                            this.rawSessionAggregatedData.femaleDwellDuration,
                                            this.rawSessionAggregatedData.maleAttentionDuration,
                                            this.rawSessionAggregatedData.femaleAttentionDuration,
                                            this.rawSessionAggregatedData.withMaskCount,
                                            this.rawSessionAggregatedData.withoutMaskCount]);
        this.clearSessionAggregatedDetections();
    } catch (error) {
        console.log("DeepsightToolkit.prototype.triggerSessionData: Catch Error: " + error);
    }
}


DeepsightToolkit.prototype.clearSessionAggregatedDetections = function () {
    var self = this;
    try {
        if (self.isAllowedToTriggerSessionAggregatedDataPerId) {
            Array.from(self._rawSessionDetections.values()).forEach(function (detection) {
                detection.finalize();
                self.emit("SessionAggregatedDetection",[detection.id,
                                                        detection.age,
                                                        detection.parseGenderToHuminizedValue(),
                                                        detection.smile,
                                                        detection.maxSmileViewing,
                                                        detection.maxSmile,
                                                        detection.detectionDuration,
                                                        detection.dwellDuration,
                                                        detection.impression,
                                                        detection.attentionDuration,
                                                        detection.viewer,
                                                        detection.faceMask]);
                });
        }
        self._rawSessionDetections.clear();
    } catch (error) {
        console.log("DeepsightToolkit.prototype.clearSessionAggregatedDetections: Catch Error: " + error);
    }
}