'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 *
 *  pullAggregatedData call a GET getRequestURL with the address: self.getaggregatedRequestURL
 *  Polling data from Deepsight Toolkit
 * 
 */
DeepsightToolkit.prototype.getAggregatedData = function () {

    try {

        var self = this;
        var headers = {};
        this.idAggregatedRequest = this.httpService.get(this.aggregatedRequestURL, headers, {
            "success": function (result) {
                setTimeout(function(){
                    try {
                        if (result !== "") {
                            var resultJSON = JSON.parse(result);
                            if((Date.now() - resultJSON.aggregator_freq) > Date.parse(resultJSON.submission_time+ " GMT")) resultJSON.aggregated_data = [];
                            self.proccesAggregatedDataRequest(resultJSON);
                        }
                        self.pullerAggregatedData();
                    } catch (error) {
                        console.log("DeepsightToolkit.prototype.getAggregatedData ------------- Error Catch success callback: " + error);
                        self.pullerAggregatedData();
                    }
                },1);
            },
            "error": function (error) {
                setTimeout(function(){
                    console.log("KO Aggregated Request");
                    console.log("DeepsightToolkit.prototype.getAggregatedData ------------- Error callback: " + error);
                    self.pullerAggregatedData();
                },1);
            }
        });
        this.timeoutAggregatedDataRequest();
    } catch (error) {
        console.log("----------- DeepsightToolkit.prototype.getAggregatedData ------------- Error Catch: " + error);
        this.timeoutAggregatedDataRequest();
    }

};


DeepsightToolkit.prototype.pullerAggregatedData = function () {
    var self = this;
    if(this.timeoutAggregatedRequestId){
        clearTimeout(self.timeoutAggregatedRequestId);
    }
    setTimeout(function(){
        self.getAggregatedData();
    }, 29000);
};


DeepsightToolkit.prototype.timeoutAggregatedDataRequest = function(){
    var self = this;
    this.timeoutAggregatedRequestId = setTimeout(function () {
            console.log("Cancel Aggregated Request");
            self.httpService.cancel(self.idAggregatedRequest);
            self.getAggregatedData();
    }, 60000);
}

/**
 *
 *  getRawData call a GET with the address: self.rawRequestURL
 *  and procces raw data from toolkit.
 * 
 *  Polling data from Deepsight Toolkit
 * 
 */
DeepsightToolkit.prototype.getRawData = function () {

    try {

        var self = this;
        var headers = {};
        this.idRequestRaw = this.httpService.get(this.rawRequestURL, headers, {
            "success": function (result) {
                    try {
                        if (result !== "") {
                            var resultJSON = JSON.parse(result);
                            if (resultJSON.people !== undefined && resultJSON.people.length)
                                self.proccesRawDataRequest(resultJSON);
                            else self.proccesRawDataRequest(new EmptyRawData());
                        }
                        self.pullerRawData();
                    } catch (error) {
                        console.log("DeepsightToolkit.prototype.pullRawData ------------- Error Catch success callback: " + error);
                        self.proccesRawDataRequest(new EmptyRawData());
                        self.pullerRawData();
                    }
            },
            "error": function (error) {
                    console.log("KO Raw Request");
                    console.log("DeepsightToolkit.prototype.pullRawData ------------- Error callback: " + error);
                    self.proccesRawDataRequest(new EmptyRawData());
                    self.pullerRawData();
            }
        });

        this.timeoutRawRequest();

    } catch (error) {

        console.log(" ----------- DeepsightToolkit.prototype.pullRawData ------------- Error Catch: " + error);
        this.timeoutRawRequest();
    }

};

DeepsightToolkit.prototype.pullerRawData = function () {
    var self = this;
    if(this.timeoutRawRequestId){
        clearTimeout(self.timeoutRawRequestId);
    }
    this.timeoutRawRequestId = setTimeout(function(){
        self.getRawData();
    }, 500);
}

DeepsightToolkit.prototype.timeoutRawRequest = function(){
    var self = this;
    this.timeoutRawRequestId = setTimeout(function () {
            self.httpService.cancel(self.idRequestRaw);
            self.getRawData();
    }, 5000);
}