'use strict';
/**
* @license
* Copyright © 2020 Sightcorp
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Sightcorp shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Sightcorp.
*/

/**
 * Deepsight Toolkit Interface Asset
 */

/**
* Inheritance on EventEmitter base class
* @type {EventEmitter}
*/

DeepsightToolkit.prototype = new EventEmitter();

/** 
 * Creates a Deepsight Toolkit Object which contains an AggregatedData Object
 * and methods to get data and process it from Deepsight Toolkit
 * @constructor
 */
function DeepsightToolkit() {

    //Required fields for managing http requests
    if (this.httpService == null) this.httpService = intuiface.get("httpService");
    this.getaggregatedRequestURL;
    this.getrawRequestURL;
    this.ipAddress                                    = "localhost";
    this.port                                         = "8080";
    this.maskThreshold                                = "85";
    this.frequencyRequest                             = 300;
    this.isAllowedToTriggerSessionAggregatedDataPerId = true;
    
    this.timeoutsRawRequestQueue           = [];
    this.timeoutsAggregatedRequestQueue    = [];
    
    this.updateURLRequests();

    // Aggregated Data object with Avg properties updated when user 
    this.aggregatedData              = new AggregatedData();
    // Aggregated Detections dictionary to keep track of new/old ids
    this._aggregatedDetections       = new Map();
    this._isRequestingAggregatedData = false;


    this.rawSessionAggregatedData = new RawSessionAggregatedData();
    this.realTimeAggregatedData   = new RealTimeAggregatedData();
    this._rawDetections           = new Map();
    this._lostDetections          = new Map();
    this._rawSessionDetections    = new Map();
    this._isRequestingRawData     = false;
    this._isRecordingSession      = false;
    
}

/**
 * Dispose resources
 */
DeepsightToolkit.prototype.dispose = function () {
};
